#include <OpenMS/FORMAT/MzMLFile.h>
#include <OpenMS/KERNEL/ConsensusMap.h>
#include <OpenMS/KERNEL/FeatureMap.h>
#include <OpenMS/FORMAT/ConsensusXMLFile.h>
#include <OpenMS/FORMAT/FeatureXMLFile.h>
#include <OpenMS/CHEMISTRY/AASequence.h>
#include <OpenMS/FORMAT/TextFile.h>

#include <set>

using namespace OpenMS;

bool testInterset(const ConvexHull2D& l, const ConvexHull2D& r)
{
  return l.getBoundingBox().intersects(r.getBoundingBox());
}

int main(int argc, char *argv[])
{

	if (argc!=4) 
	{
    std::cerr << "Mark all overlapping feature and mark them in the consensusXML\nUsage " << argv[0] << " <in_SIM_featureXML> <in_SIM_ConsensusMap.consensusXML> <out_annotated_SIM_ConsensusMap.consensusXML>\n";
		return 0;
	}
	
  ConsensusXMLFile cf;
	ConsensusMap cm;
  cf.load(String(argv[2]), cm);

  FeatureXMLFile ff;
  FeatureMap<> fm;
  ff.load(String(argv[1]), fm);

  fm.updateUniqueIdToIndex();
  cm.updateUniqueIdToIndex();

  std::set<UInt64> overlapping_features;

  // iterate over all pairs of features
  for(Size i = 0 ; i < fm.size() ; ++i)
  {
    Feature& fi = fm[i];
    for(Size j = i + 1 ; j < fm.size() ; ++j)
    {
      Feature& fj = fm[j];
      if(testInterset(fi.getConvexHull(), fj.getConvexHull()))
      {
        if(fi.getCharge() == fj.getCharge())
        {
          if(std::abs(std::abs(fi.getRT() - fj.getRT()) - 0.1) < 0.000001)
          {
            if(std::abs(std::abs(fi.getMZ() - fj.getMZ()) - (6.0/fi.getCharge())) < 0.000001)
            {
              continue;
            }
          }
        }
        overlapping_features.insert(fi.getUniqueId());
        overlapping_features.insert(fj.getUniqueId());
      }
    }
  }

  // mark consensus features as potentially overlapping
  for(ConsensusMap::iterator cm_iter = cm.begin() ; cm_iter != cm.end() ; ++cm_iter)
  {
    // check if it has overlapping features
    ConsensusFeature& cf = *cm_iter;
    ConsensusFeature::HandleSetType fhs = cf.getFeatures();
    if(fhs.size() != 2) continue; // this should not happen


    FeatureHandle f1 = *(fhs.begin());
    FeatureHandle f2 = *(++fhs.begin());

    if(
        overlapping_features.count(f1.getUniqueId())
        ||
        overlapping_features.count(f2.getUniqueId())
        )
    {
      // mark feature as potentially overlapping
      cf.setMetaValue("has_overlapping_feature", 1);
    }
    else
    {
      // mark feature as non-overlapping
      cf.setMetaValue("has_overlapping_feature", 0);
    }
  }

  cf.store(String(argv[3]), cm);

	return 0;
}
